@@ -1,7 +1,7 @@
 use strict;
 package HTTP::BrowserDetect;
 BEGIN {
-  $HTTP::BrowserDetect::VERSION = '1.21';
+  $HTTP::BrowserDetect::VERSION = '1.23';
 }
 
 use vars qw(@ISA @EXPORT @EXPORT_OK @ALL_TESTS);
@@ -30,6 +30,7 @@ push @ALL_TESTS, qw(
     freebsd bsd         vms
     x11     amiga       android
     win7    ps3gameos   pspgameos
+    wince
 );
 
 # Devices
@@ -61,6 +62,12 @@ push @ALL_TESTS, qw(
     elinks
 );
 
+# Firefox variants
+push @ALL_TESTS, qw(
+    firebird    iceweasel   phoenix 
+    namoroka
+);
+
 # Robots
 push @ALL_TESTS, qw(
     puf         curl        wget
@@ -163,7 +170,7 @@ sub _test {
     $self->{tests} = {};
     my $tests = $self->{tests};
 
-    my @ff = qw( firefox firebird iceweasel phoenix );
+    my @ff = qw( firefox firebird iceweasel phoenix namoroka );
     my $ff = join "|", @ff;
 
     my $ua = lc $self->{user_agent};
@@ -240,7 +247,7 @@ sub _test {
     # Mozilla browsers
 
     $tests->{GECKO} = ( index( $ua, "gecko" ) != -1 )
-        && ( index( $ua, "khtml, like gecko" ) == -1 );
+        && ( index( $ua, "like gecko" ) == -1 );
 
     $tests->{CHROME} = ( index( $ua, "chrome/" ) != -1 );
     $tests->{SAFARI}
@@ -288,6 +295,7 @@ sub _test {
     $tests->{NETSCAPE}
         = (    !$tests->{FIREFOX}
             && !$tests->{SAFARI}
+            && !$tests->{CHROME}
             && index( $ua, "mozilla" ) != -1
             && index( $ua, "spoofer" ) == -1
             && index( $ua, "compatible" ) == -1
@@ -401,7 +409,7 @@ sub _test {
     $tests->{MSN} = ( (index( $ua, "msnbot" ) != -1 || index( $ua, "bingbot" )) != -1 );
     $tests->{MSNMOBILE} = ( (index( $ua, "msnbot-mobile" ) != -1 || index( $ua, "bingbot-mobile" )) != -1 );
     $tests->{JAVA}
-        = ( index( $ua, "java" ) != -1 || index( $ua, "jdk" ) != -1 );
+        = ( index( $ua, "java" ) != -1 || index( $ua, "jdk" ) != -1 || index($ua, "jakarta commons-httpclient") != -1);
     $tests->{ALTAVISTA}    = ( index( $ua, "altavista" ) != -1 );
     $tests->{SCOOTER}      = ( index( $ua, "scooter" ) != -1 );
     $tests->{LYCOS}        = ( index( $ua, "lycos" ) != -1 );
@@ -427,6 +435,7 @@ sub _test {
                 || $tests->{MSN}
                 || $tests->{MSNMOBILE}
                 || $tests->{FACEBOOK}
+                || $tests->{JAVA}
         )
             || index( $ua, "bot" ) != -1
             || index( $ua, "spider" ) != -1
@@ -552,6 +561,8 @@ sub _test {
     $tests->{WIN7}     = ( index( $ua, "nt 6.1" ) != -1 );
     $tests->{DOTNET}   = ( index( $ua, ".net clr" ) != -1 );
 
+    $tests->{WINCE} = ( index( $ua, "windows ce" ) != -1 );
+
     $tests->{WINME} = ( index( $ua, "win 9x 4.90" ) != -1 );    # whatever
     $tests->{WIN32} = (
         (          $tests->{WIN95}
@@ -579,6 +590,7 @@ sub _test {
                 || $tests->{WINVISTA}
                 || $tests->{WIN7}
                 || $tests->{WINME}
+                || $tests->{WINCE}
         )
             || index( $ua, "win" ) != -1
     );
@@ -1070,6 +1082,21 @@ sub _format_minor {
 
 }
 
+sub browser_properties {
+
+    my ( $self, $check ) = _self_or_default( @_ );
+
+    my @browser_properties;
+    foreach my $property (keys %{$self->{tests}}) {
+        push @browser_properties, lc($property) if (${$self->{tests}}{$property});
+    }
+
+    # devices are a property too but it's not stored in %tests 
+    # so I explicitly test for it and add it
+    push @browser_properties, 'device' if ($self->device());
+
+    return @browser_properties;
+}
 1;
 
 
@@ -1082,7 +1109,7 @@ HTTP::BrowserDetect - Determine Web browser, version, and platform from an HTTP
 
 =head1 VERSION
 
-version 1.21
+version 1.23
 
 =head1 SYNOPSIS
 
@@ -1178,6 +1205,12 @@ are subject to change and are really meant for display purposes.  You should
 use the device() method in your logic.  Returns one of: BlackBerry, iPhone,
 iPod or iPad.  Returns UNDEF if no hardware can be detected.
 
+=head2 browser_properties()
+
+Returns a list of the browser properties, that is, all of the tests that passed
+for the provided user_agent string. Operating systems, devices, browser names, 
+mobile and robots are all browser properties.
+
 =head1 Detecting Browser Version
 
 Please note that that the version(), major() and minor() methods have been
@@ -1281,6 +1314,7 @@ winnt, which is a type of win32)
         winme win95 win98
         winnt
             win2k winxp win2k3 winvista win7
+    wince
 
 =head2 dotnet()
 
@@ -1522,6 +1556,8 @@ Graham Barr
 
 Enrico Sorcinelli
 
+Olivier Bilodeau
+
 =head1 TO DO
 
 The _engine() method currently only handles Gecko.  It needs to be expanded to
@@ -1609,7 +1645,7 @@ Olaf Alders <olaf@wundercounter.com> (current maintainer)
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Lee Semel.
+This software is copyright (c) 2011 by Lee Semel.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
